package dev.inmo.postssystem.features.auth.client

import dev.inmo.micro_utils.fsm.common.CheckableHandlerHolder
import dev.inmo.micro_utils.fsm.common.StatesMachine
import dev.inmo.micro_utils.fsm.common.dsl.FSMBuilder
import dev.inmo.postssystem.features.auth.client.settings.AuthSettings
import dev.inmo.postssystem.features.auth.client.ui.AuthUIFSMState
import dev.inmo.postssystem.features.common.common.ui.fsm.UIFSMHandler
import dev.inmo.postssystem.features.common.common.ui.fsm.UIFSMState
import org.koin.core.Koin
import org.koin.core.parameter.ParametersHolder
import org.koin.core.qualifier.Qualifier
import org.koin.core.scope.Scope
import kotlin.reflect.KClass

// Костыль, в JS на момент Пн дек  6 14:19:29 +06 2021 если использовать strictlyOn генерируются
// некорректные безымянные классы (у них отсутствует метод handleState)
class DefaultStateHandlerWrapper<T : UIFSMState>(
    private val klass: KClass<out UIFSMHandler<T>>,
    private val koin: Koin,
    private val stateKlass: KClass<T>,
    private val qualifier: Qualifier? = null,
    private val parameters: ((T) -> ParametersHolder)? = null
) : CheckableHandlerHolder<UIFSMState, UIFSMState> {
    override suspend fun StatesMachine<in UIFSMState>.handleState(state: UIFSMState): UIFSMState? {
        @Suppress("UNCHECKED_CAST", "NAME_SHADOWING")
        val state = state as T
        return runCatching {
            val authSettings = koin.get<AuthSettings>()
            authSettings.loadingJob.join()
            if (authSettings.authorizedDIModule.value == null) {
                error("Can't perform state $state: Auth module was not initialized")
            } else {
                koin.get<UIFSMHandler<T>>(klass, qualifier, parameters ?.let { { it(state) } }).run {
                    handleState(state)
                }
            }
        }.getOrElse { e ->
            e.printStackTrace()
            AuthUIFSMState(state)
        }
    }

    override suspend fun checkHandleable(state: UIFSMState): Boolean = stateKlass.isInstance(state)
}
inline fun <reified T : UIFSMState> FSMBuilder<UIFSMState>.registerAfterAuthHandler(
    koin: Koin,
    klass: KClass<out UIFSMHandler<T>>,
    qualifier: Qualifier? = null,
    noinline parameters: ((T) -> ParametersHolder)? = null
) = add(
    DefaultStateHandlerWrapper<T>(
        klass,
        koin,
        T::class,
        qualifier,
        parameters
    )
)
