package dev.inmo.postssystem.features.common.common.ui.fsm

import dev.inmo.micro_utils.common.onPresented
import dev.inmo.micro_utils.fsm.common.*
import dev.inmo.micro_utils.fsm.common.dsl.FSMBuilder
import org.koin.core.scope.Scope

interface UIFSMHandler<T : UIFSMState> : StatesHandler<T, UIFSMState> {
    fun interface Registrator {
        fun FSMBuilder<UIFSMState>.include()
    }
    val defaultExceptionsHandlers: Iterable<UIFSMExceptionHandler>

    suspend fun StatesMachine<in UIFSMState>.safeHandleState(state: T): UIFSMState?
    override suspend fun StatesMachine<in UIFSMState>.handleState(state: T): UIFSMState? {
        return runCatching {
            safeHandleState(state).also(::println)
        }.getOrElse {
            errorToNextStep(state, it) ?.let { return it } ?: throw it
        }.also(::println)
    }

    suspend fun errorToNextStep(
        currentState: T,
        e: Throwable
    ): UIFSMState? {
        defaultExceptionsHandlers.forEach {
            it.handle(currentState, e).onPresented { state ->
                return state
            }
        }
        return null
    }
}
