package dev.inmo.postssystem.targets.telegram.publication.server

import dev.inmo.micro_utils.mime_types.KnownMimeTypes
import dev.inmo.postssystem.features.content.common.BinaryContent
import dev.inmo.postssystem.features.content.text.common.TextContent
import dev.inmo.postssystem.features.publication.server.PublicationPost
import dev.inmo.postssystem.features.publication.server.PublicationTarget
import dev.inmo.tgbotapi.bot.TelegramBot
import dev.inmo.tgbotapi.requests.abstracts.MultipartFile
import dev.inmo.tgbotapi.requests.send.SendTextMessage
import dev.inmo.tgbotapi.requests.send.media.*
import dev.inmo.tgbotapi.types.ChatId
import kotlinx.coroutines.delay

class PublicationTargetTelegram(
    private val bot: TelegramBot,
    private val targetChatId: ChatId
) : PublicationTarget {
    override suspend fun publish(post: PublicationPost) {
        post.content.mapNotNull {
            when (val content = it.content) {
                is BinaryContent -> {
                    val multipartFile by lazy {
                        MultipartFile(content.filename.name) {
                            content.inputProvider()
                        }
                    }
                    when (content.mimeType) {
                        is KnownMimeTypes.Image.Jpeg,
                        is KnownMimeTypes.Image.Png -> {
                            SendPhoto(targetChatId, multipartFile)
                        }
                        is KnownMimeTypes.Video.Mp4 -> {
                            SendVideo(targetChatId, multipartFile)
                        }
                        is KnownMimeTypes.Audio.Mpeg -> {
                            SendAudio(targetChatId, multipartFile)
                        }
                        else -> SendDocument(
                            targetChatId,
                            multipartFile
                        )
                    }
                }
                is TextContent -> {
                    SendTextMessage(targetChatId, content.text)
                }
                else -> null
            }
        }.forEach { request ->
            repeat(3) {
                runCatching {
                    bot.execute(request)
                }.onFailure {
                    it.printStackTrace()
                }.onSuccess {
                    return@forEach
                }
                delay(1000L)
            }
            delay(100L)
        }
    }
}
